@@ -16,13 +16,19 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "ephy-gui.h"
 #include "eel-gconf-extensions.h"
 
 #include <ctype.h>
 #include <string.h>
-#include <libgnome/gnome-i18n.h>
+#include <bonobo/bonobo-i18n.h>
+#include <libgnome/gnome-help.h>
 #include <gtk/gtktreemodel.h>
+#include <gtk/gtkmessagedialog.h>
 
 /* Styles for tab labels */
 GtkStyle *loading_text_style = NULL;
@@ -146,3 +152,29 @@ ephy_gui_confirm_overwrite_file (GtkWidget *parent, const char *filename)
 
 	return res;
 }
+
+void		
+ephy_gui_help (GtkWindow *parent,
+	       const char *file_name,
+               const char *link_id)
+{
+	GError *err = NULL;
+
+	gnome_help_display (file_name, link_id, &err);
+
+	if (err != NULL)
+	{
+		GtkWidget *dialog;
+		dialog = gtk_message_dialog_new (parent,
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_OK,
+						 _("Could not display help: %s"), err->message);
+		g_signal_connect (G_OBJECT (dialog), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		gtk_widget_show (dialog);
+		g_error_free (err);
+	}
+}
